@@ -1,3 +1,4 @@
+from django.conf import settings
 from django.db import models, transaction
 from django.db.models import F, Q
 from django.urls import reverse
@@ -15,7 +16,16 @@ from obapi.converters import (
     SpotifyEpisodeURLConverter,
     YoutubeVideoURLConverter,
 )
-from obapi.models import Author, ExternalLink, Idea, Tag, Topic
+from obapi.models import (
+    CLASSIFIER_SLUG_MAX_LENGTH,
+    Author,
+    ExternalLink,
+    Idea,
+    Tag,
+    Topic,
+)
+
+DOWNLOAD_BATCH_SIZE = getattr(settings, "OBAPI_DOWNLOAD_BATCH_SIZE", 10000)
 
 
 class ContentItemQuerySet(InheritanceQuerySet):
@@ -64,14 +74,14 @@ class ContentItemQuerySet(InheritanceQuerySet):
         if author_names is None:
             return None
         with transaction.atomic():
-            # Match by alias or slug
+            # Match by alias
             authors = [
-                Author.objects.filter(
-                    Q(alias__text__iexact=author_name)
-                    | Q(slug=utils.to_slug(author_name))
+                Author.objects.get_or_create(
+                    alias__text=utils.to_slug(
+                        author_name, max_length=CLASSIFIER_SLUG_MAX_LENGTH
+                    ),
+                    defaults={"name": author_name},
                 )
-                .distinct()
-                .get_or_create(defaults={"name": author_name})
                 for author_name in author_names
             ]
             return [author[0] for author in authors]
@@ -85,9 +95,11 @@ class ContentItemQuerySet(InheritanceQuerySet):
         tags = []
         with transaction.atomic():
             for classifier_name in classifier_names:
-                # Match by alias or slug
-                query = Q(alias__text__iexact=classifier_name) | Q(
-                    slug=utils.to_slug(classifier_name)
+                # Match by alias
+                query = Q(
+                    alias__text=utils.to_slug(
+                        classifier_name, max_length=CLASSIFIER_SLUG_MAX_LENGTH
+                    )
                 )
                 try:
                     ideas.append(Idea.objects.get(query))
@@ -103,9 +115,7 @@ class ContentItemQuerySet(InheritanceQuerySet):
                 # If that fails, try to get a tag with alias = classifier_name
                 # If no such tag exists, create a new one
                 tags.append(
-                    Tag.objects.filter(query)
-                    .distinct()
-                    .get_or_create(
+                    Tag.objects.filter(query).get_or_create(
                         defaults={"name": classifier_name},
                     )[0]
                 )
@@ -157,6 +167,19 @@ class ContentItemQuerySet(InheritanceQuerySet):
                     created_items.append(None)
         return created_items
 
+    def bulk_create_items(self, item_ids, batch_size=DOWNLOAD_BATCH_SIZE):
+        """Create items from their IDs, in batches.
+
+        Returns
+        -------
+        The number of created items.
+        """
+        created_count = 0
+        for item_ids_chunk in utils.chunk_iterator(item_ids, batch_size):
+            created_items = self.create_items(item_ids_chunk)
+            created_count += sum(item is not None for item in created_items)
+        return created_count
+
     def update_items(self, exclude=None):
         """Update items in QuerySet, excluding certain fields.
 
@@ -242,9 +265,16 @@ class ContentItem(models.Model):
     objects = ContentItemQuerySet.as_manager()
 
     create_timestamp = models.DateTimeField("creation date", auto_now_add=True)
-    update_timestamp = models.DateTimeField("update date", auto_now=True)
+    update_timestamp = models.DateTimeField(
+        "update date",
+        auto_now=True,
+        help_text="When the item was last updated - either by hand or automatically.",
+    )
     download_timestamp = models.DateTimeField(
-        "download date", editable=False, blank=True
+        "download date",
+        editable=False,
+        null=True,
+        help_text="When the item was last downloaded.",
     )
 
     title = models.CharField(max_length=100, help_text="Title of content.")
@@ -376,7 +406,7 @@ class YoutubeContentItem(VideoContentItem):
     def site_url(self):
         return "https://www.youtube.com/"
 
-    @property
+    @classmethod
     def site_name(self):
         return "YouTube"
 
@@ -425,7 +455,7 @@ class SpotifyContentItem(AudioContentItem):
     def site_url(self):
         return "https://open.spotify.com/"
 
-    @property
+    @classmethod
     def site_name(self):
         return "Spotify"
 
@@ -452,17 +482,21 @@ class OBContentItemQuerySet(ContentItemQuerySet):
         (OBPostShortURLConverter(), "ob_post_number"),
     )
 
-    def download_new_items(self, min_publish_date=None):
+    def download_new_items(self, min_edit_date=None):
         """Add posts whose names are not found in the database.
 
         Do not return items which were not successfully created.
         """
-        if min_publish_date is None:
+        self.update_last_edit_dates()
+        if min_edit_date is None:
             try:
-                # Ignore names of posts published before first download date
-                min_publish_date = self.earliest(
-                    "download_timestamp"
-                ).download_timestamp
+                # Take most recent edit date, among posts which haven't been edited
+                # since creation
+                min_edit_date = (
+                    self.filter(edit_date__lt=F("create_timestamp"))
+                    .latest("edit_date")
+                    .edit_date
+                )
             except self.model.DoesNotExist:
                 # No items
                 pass
@@ -470,13 +504,13 @@ class OBContentItemQuerySet(ContentItemQuerySet):
         site_names = [
             name
             for name, date in assemble_ob_edit_dates().items()
-            if min_publish_date is None or date > min_publish_date
+            if min_edit_date is None or date > min_edit_date
         ]
         db_names = self.values_list("item_id", flat=True)
-        names_to_add = [name for name in site_names if name not in db_names]
+        missing_names = [name for name in site_names if name not in db_names]
 
-        created_items = self.create_items(names_to_add)
-        return [item for item in created_items if item is not None]
+        created_item_count = self.bulk_create_items(missing_names)
+        return created_item_count
 
     def update_edited_items(self):
         """Update posts with unsaved edits."""
@@ -486,7 +520,10 @@ class OBContentItemQuerySet(ContentItemQuerySet):
         return [result[0] for result in updated_items]
 
     def update_last_edit_dates(self):
-        """Synchronise edit dates with the overcomingbias site."""
+        """Synchronise edit dates with the overcomingbias site.
+
+        This uses the bulk_update method, so does not change the update_timestamp field.
+        """
         edit_dates = assemble_ob_edit_dates()
 
         all_items = self.all()
@@ -509,7 +546,7 @@ class OBContentItem(TextContentItem):
         "number ID", unique=True, help_text="Post number identifier."
     )
     disqus_id = models.CharField(
-        max_length=200, unique=True, help_text="Post Disqus API string identifier."
+        max_length=200, blank=True, help_text="Post Disqus API string identifier."
     )
     ob_likes = models.PositiveSmallIntegerField(
         "likes", blank=True, null=True, help_text="Number of likes."
@@ -529,9 +566,9 @@ class OBContentItem(TextContentItem):
     def site_url(self):
         return "https://www.overcomingbias.com/"
 
-    @property
+    @classmethod
     def site_name(self):
-        return "Overcoming Bias"
+        return "OvercomingBias"
 
     class Meta:
         verbose_name = "overcomingbias post"
